import { AppConfiguration } from './AppConfiguration';
import { IModuleBuilder } from './IModuleBuilder';
import { IContainer, Token, Type, LoadType, IContainerBuilder, DefaultContainerBuilder } from '@ts-ioc/core';
import { CustomRegister, IApplicationBuilder } from './IApplicationBuilder';
/**
 * application builder.
 *
 * @export
 * @class ApplicationBuilder
 * @extends {ModuleBuilder<T>}
 * @template T
 */
export declare class ApplicationBuilder<T> implements IApplicationBuilder<T> {
    baseURL: string;
    private moduleBuilder;
    private container;
    private builder;
    protected globalConfig: Promise<AppConfiguration<T>>;
    protected usedModules: LoadType[];
    protected customRegs: CustomRegister<T>[];
    constructor(baseURL?: string);
    /**
     * get container
     *
     * @returns
     * @memberof ApplicationBuilder
     */
    getContainer(): IContainer;
    /**
     * set container.
     *
     * @param {IContainer} container
     * @returns
     * @memberof ApplicationBuilder
     */
    setContainer(container: IContainer): this;
    /**
     * get container builder.
     *
     * @returns
     * @memberof ModuleBuilder
     */
    getContainerBuilder(): IContainerBuilder;
    /**
     * use container builder
     *
     * @param {IContainerBuilder} builder
     * @returns
     * @memberof ModuleBuilder
     */
    setContainerBuilder(builder: IContainerBuilder): this;
    /**
     * get module builer.
     *
     * @returns {IModuleBuilder<T>}
     * @memberof IApplicationBuilder
     */
    getModuleBuilder(): IModuleBuilder<T>;
    /**
     * set module builder.
     *
     * @param {IModuleBuilder<T>} builder
     * @returns {this}
     * @memberof ApplicationBuilder
     */
    setModuleBuilder(builder: IModuleBuilder<T>): this;
    /**
     * use configuration.
     *
     * @param {(string | AppConfiguration<T>)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    useConfiguration(config?: string | AppConfiguration<T>): this;
    /**
     * use module, custom module.
     *
     * @param {...(LoadType | CustomRegister<T>)[]} modules
     * @returns {this}
     * @memberof PlatformServer
     */
    use(...modules: LoadType[]): this;
    /**
     * register modules via custom.
     *
     * @param {...CustomRegister<T>[]} moduleRegs
     * @returns {this}
     * @memberof ApplicationBuilder
     */
    registerModules(...moduleRegs: CustomRegister<T>[]): this;
    /**
     * build and bootstrap application.
     *
     * @param {(Token<T> | Type<any> | AppConfiguration<T>)} token
     * @returns {Promise<T>}
     * @memberof ApplicationBuilder
     */
    bootstrap(token: Token<T> | Type<any> | AppConfiguration<T>): Promise<any>;
    /**
     * build application.
     *
     * @param {IModuleBuilder<T>} builder
     * @param {(Token<T> | Type<any> | AppConfiguration<T>)} token
     * @returns {Promise<any>}
     * @memberof ApplicationBuilder
     */
    build(token: Token<T> | Type<any> | AppConfiguration<T>): Promise<any>;
    protected createInstance(builder: IModuleBuilder<T>, config: AppConfiguration<T>): Promise<any>;
    /**
     * create default module builder.
     *
     * @protected
     * @returns
     * @memberof ApplicationBuilder
     */
    protected createModuleBuilder(): IModuleBuilder<any>;
    /**
     * create default container builder.
     *
     * @protected
     * @returns
     * @memberof ApplicationBuilder
     */
    protected createContainerBuilder(): DefaultContainerBuilder;
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @returns {Promise<IContainer>}
     * @memberof ApplicationBuilder
     */
    protected registerExts(container: IContainer): Promise<IContainer>;
    protected initContainer(config: AppConfiguration<T>, container: IContainer): Promise<IContainer>;
    protected bindConfiguration(container: IContainer, config: AppConfiguration<T>): void;
    /**
     * meger config configuration with global config.
     *
     * @protected
     * @param {AppConfiguration<T>} cfg
     * @returns {Promise<AppConfiguration<T>>}
     * @memberof ApplicationBuilder
     */
    protected mergeConfigure(cfg: AppConfiguration<T>): Promise<AppConfiguration<T>>;
    protected getDefaultConfig(): AppConfiguration<T>;
    static classAnnations: any;
}
