import { IContainer, Token, Type, Providers } from '@ts-ioc/core';
import { IModuleBuilder } from './IModuleBuilder';
import { ModuleConfiguration } from './ModuleConfiguration';
/**
 * server app bootstrap
 *
 * @export
 * @class ModuleBuilder
 */
export declare class ModuleBuilder<T> implements IModuleBuilder<T> {
    /**
     * ioc container
     *
     * @type {IContainer}
     * @memberof ModuleBuilder
     */
    container: IContainer;
    constructor();
    /**
     * build module.
     *
     * @param {(Token<T>| ModuleConfiguration<T>)} [token]
     * @returns {Promise<any>}
     * @memberof ModuleBuilder
     */
    build(token: Token<T> | Type<any> | ModuleConfiguration<T>, data?: any): Promise<T>;
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {ModuleConfiguration<T>} config
     * @returns {Promise<T>}
     * @memberof IModuleBuilder
     */
    buildStrategy(instance: T, config: ModuleConfiguration<T>): Promise<T>;
    getBuilder(config: ModuleConfiguration<T>): IModuleBuilder<T>;
    getDecorator(): string;
    /**
     * get configuration.
     *
     * @returns {ModuleConfiguration<T>}
     * @memberof ModuleBuilder
     */
    getConfigure(token?: Token<any> | ModuleConfiguration<T>): ModuleConfiguration<T>;
    createInstance(token: Token<T>, cfg: ModuleConfiguration<T>, data?: any): Promise<T>;
    protected getBootstrapToken(cfg: ModuleConfiguration<T>, token?: Token<T> | Type<any>): Token<T>;
    protected getBuilderViaConfig(builder: Token<IModuleBuilder<T>> | IModuleBuilder<T>): IModuleBuilder<T>;
    protected getBuilderViaToken(mdl: Token<T>): IModuleBuilder<T>;
    protected getMetaConfig(bootModule: Type<any>): ModuleConfiguration<T>;
    protected registerDepdences(config: ModuleConfiguration<T>): Promise<IContainer>;
    protected bindProvider(container: IContainer, providers: Providers[]): void;
    static classAnnations: any;
}
