"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var IModuleBuilder_1 = require("./IModuleBuilder");
var decorators_1 = require("./decorators");
/**
 * server app bootstrap
 *
 * @export
 * @class ModuleBuilder
 */
var ModuleBuilder = /** @class */ (function () {
    function ModuleBuilder() {
    }
    ModuleBuilder_1 = ModuleBuilder;
    /**
     * build module.
     *
     * @param {(Token<T>| ModuleConfiguration<T>)} [token]
     * @returns {Promise<any>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.build = function (token, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var cfg, buider, instacnce;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cfg = this.getConfigure(token);
                        buider = this.getBuilder(cfg);
                        return [4 /*yield*/, buider.createInstance(core_1.isToken(token) ? token : null, cfg, data)];
                    case 1:
                        instacnce = _a.sent();
                        return [4 /*yield*/, buider.buildStrategy(instacnce, cfg)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, instacnce];
                }
            });
        });
    };
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {ModuleConfiguration<T>} config
     * @returns {Promise<T>}
     * @memberof IModuleBuilder
     */
    ModuleBuilder.prototype.buildStrategy = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, instance];
            });
        });
    };
    ModuleBuilder.prototype.getBuilder = function (config) {
        var builder;
        if (config.builder) {
            builder = this.getBuilderViaConfig(config.builder);
        }
        else {
            var token = this.getBootstrapToken(config);
            if (token) {
                builder = this.getBuilderViaToken(token);
            }
        }
        return builder || this;
    };
    ModuleBuilder.prototype.getDecorator = function () {
        return decorators_1.DIModule.toString();
    };
    /**
     * get configuration.
     *
     * @returns {ModuleConfiguration<T>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.getConfigure = function (token) {
        var cfg;
        if (core_1.isClass(token)) {
            cfg = this.getMetaConfig(token);
        }
        else if (core_1.isToken(token)) {
            var tokenType = this.container.getTokenImpl(token);
            if (core_1.isClass(tokenType)) {
                cfg = this.getMetaConfig(tokenType);
            }
        }
        else {
            cfg = token;
            var bootToken = this.getBootstrapToken(cfg);
            var typeTask = core_1.isClass(bootToken) ? bootToken : this.container.getTokenImpl(bootToken);
            if (core_1.isClass(typeTask)) {
                cfg = core_1.lang.assign({}, this.getMetaConfig(typeTask), cfg || {});
            }
        }
        return cfg || {};
    };
    ModuleBuilder.prototype.createInstance = function (token, cfg, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var bootToken;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        bootToken = this.getBootstrapToken(cfg, token);
                        if (!bootToken) {
                            throw new Error('not find bootstrap token.');
                        }
                        return [4 /*yield*/, this.registerDepdences(cfg)];
                    case 1:
                        _a.sent();
                        if (core_1.isClass(bootToken)) {
                            if (!this.container.has(bootToken)) {
                                this.container.register(bootToken);
                            }
                            return [2 /*return*/, this.container.resolve(bootToken)];
                        }
                        else {
                            return [2 /*return*/, this.container.resolve(bootToken)];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    ModuleBuilder.prototype.getBootstrapToken = function (cfg, token) {
        return cfg.bootstrap || token;
    };
    ModuleBuilder.prototype.getBuilderViaConfig = function (builder) {
        if (core_1.isToken(builder)) {
            return this.container.resolve(builder);
        }
        else if (builder instanceof ModuleBuilder_1) {
            return builder;
        }
        return null;
    };
    ModuleBuilder.prototype.getBuilderViaToken = function (mdl) {
        if (core_1.isToken(mdl)) {
            var taskType = core_1.isClass(mdl) ? mdl : this.container.getTokenImpl(mdl);
            if (taskType) {
                var meta = core_1.lang.first(core_1.getTypeMetadata(this.getDecorator(), taskType));
                if (meta && meta.builder) {
                    return core_1.isToken(meta.builder) ? this.container.resolve(meta.builder) : meta.builder;
                }
            }
        }
        return null;
    };
    ModuleBuilder.prototype.getMetaConfig = function (bootModule) {
        var decorator = this.getDecorator();
        if (core_1.hasClassMetadata(decorator, bootModule)) {
            var metas = core_1.getTypeMetadata(decorator, bootModule);
            if (metas && metas.length) {
                var meta = metas[0];
                meta.bootstrap = meta.bootstrap || bootModule;
                return core_1.lang.omit(meta, 'builder');
            }
        }
        return null;
    };
    ModuleBuilder.prototype.registerDepdences = function (config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 2];
                        return [4 /*yield*/, (_a = this.container).loadModule.apply(_a, config.imports)];
                    case 1:
                        _b.sent();
                        _b.label = 2;
                    case 2:
                        if (core_1.isArray(config.providers) && config.providers.length) {
                            this.bindProvider(this.container, config.providers);
                        }
                        return [2 /*return*/, this.container];
                }
            });
        });
    };
    ModuleBuilder.prototype.bindProvider = function (container, providers) {
        providers.forEach(function (p, index) {
            if (core_1.isUndefined(p) || core_1.isNull(p)) {
                return;
            }
            if (core_1.isProviderMap(p)) {
                p.forEach(function (k, f) {
                    container.bindProvider(k, f);
                });
            }
            else if (p instanceof core_1.Provider) {
                container.bindProvider(p.type, function () {
                    var providers = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        providers[_i] = arguments[_i];
                    }
                    return p.resolve.apply(p, [container].concat(providers));
                });
            }
            else if (core_1.isClass(p)) {
                if (!container.has(p)) {
                    container.register(p);
                }
            }
            else if (core_1.isBaseObject(p)) {
                var pr_1 = p;
                var isobjMap = false;
                if (core_1.isToken(pr_1.provide)) {
                    if (core_1.isArray(pr_1.deps) && pr_1.deps.length) {
                        pr_1.deps.forEach(function (d) {
                            if (core_1.isClass(d) && !container.has(d)) {
                                container.register(d);
                            }
                        });
                    }
                    if (!core_1.isUndefined(pr_1.useValue)) {
                        container.bindProvider(pr_1.provide, function () { return pr_1.useValue; });
                    }
                    else if (core_1.isClass(pr_1.useClass)) {
                        if (!container.has(pr_1.useClass)) {
                            container.register(pr_1.useClass);
                        }
                        container.bindProvider(pr_1.provide, pr_1.useClass);
                    }
                    else if (core_1.isFunction(pr_1.useFactory)) {
                        container.bindProvider(pr_1.provide, function () {
                            var args = [];
                            if (core_1.isArray(pr_1.deps) && pr_1.deps.length) {
                                args = pr_1.deps.map(function (d) {
                                    if (core_1.isClass(d)) {
                                        return container.get(d);
                                    }
                                    else {
                                        return d;
                                    }
                                });
                            }
                            return pr_1.useFactory.apply(pr_1, args);
                        });
                    }
                    else if (core_1.isToken(pr_1.useExisting)) {
                        if (container.has(pr_1.useExisting)) {
                            container.bindProvider(pr_1.provide, pr_1.useExisting);
                        }
                        else {
                            console.log('has not register:', pr_1.useExisting);
                        }
                    }
                    else {
                        isobjMap = true;
                    }
                }
                else {
                    isobjMap = true;
                }
                if (isobjMap) {
                    core_1.lang.forIn(p, function (val, name) {
                        if (!core_1.isUndefined(val)) {
                            if (core_1.isClass(val)) {
                                container.bindProvider(name, val);
                            }
                            else if (core_1.isFunction(val) || core_1.isString(val)) {
                                container.bindProvider(name, function () { return val; });
                            }
                            else {
                                container.bindProvider(name, val);
                            }
                        }
                    });
                }
            }
            else if (core_1.isFunction(p)) {
                container.bindProvider(name, function () { return p; });
            }
        });
    };
    ModuleBuilder.classAnnations = { "name": "ModuleBuilder", "params": { "constructor": [], "build": ["token", "data"], "buildStrategy": ["instance", "config"], "getBuilder": ["config"], "getDecorator": [], "getConfigure": ["token"], "createInstance": ["token", "cfg", "data"], "getBootstrapToken": ["cfg", "token"], "getBuilderViaConfig": ["builder"], "getBuilderViaToken": ["mdl"], "getMetaConfig": ["bootModule"], "registerDepdences": ["config"], "bindProvider": ["container", "providers"] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], ModuleBuilder.prototype, "container", void 0);
    ModuleBuilder = ModuleBuilder_1 = tslib_1.__decorate([
        core_1.Injectable(IModuleBuilder_1.ModuleBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], ModuleBuilder);
    return ModuleBuilder;
    var ModuleBuilder_1;
}());
exports.ModuleBuilder = ModuleBuilder;

//# sourceMappingURL=sourcemaps/ModuleBuilder.js.map
