import { ClassMetadata, Token, MetadataAdapter, MetadataExtends, ITypeDecorator, Registration } from '@ts-ioc/core';
import { IModuleBuilder } from '../IModuleBuilder';
import { IApplication } from '../IApplication';
import { ModuleConfiguration } from '../ModuleConfiguration';
/**
 * DI module metadata.
 *
 * @export
 * @interface DIModuleMetadata
 * @extends {ModuleConfiguration<any>}
 * @extends {ClassMetadata}
 */
export interface DIModuleMetadata extends ModuleConfiguration<any>, ClassMetadata {
    /**
     * custom decorator type.
     *
     * @type {string}
     * @memberof DIModuleMetadata
     */
    decorType?: string;
}
/**
 * DIModule decorator, use to define class is a task element.
 *
 * @export
 * @interface IDIModuleDecorator
 * @extends {ITypeDecorator<T>}
 * @template T
 */
export interface IDIModuleDecorator<T extends DIModuleMetadata> extends ITypeDecorator<T> {
    /**
     * DIModule decorator, use to define class as DI Module.
     *
     * @DIModule
     *
     * @param {T} [metadata] bootstrap metadate config.
     */
    (metadata?: T): ClassDecorator;
    /**
     * DIModule decorator, use to define class as DI Module.
     *
     * @DIModule
     * @param {string} provide application name or provide.
     * @param {string} [alias] application alias name.
     */
    (provide: Registration<any> | symbol | string, alias?: string): ClassDecorator;
    /**
     * DIModule decorator, use to define class as DI Module.
     *
     * @DIModule
     * @param {string} provide application name or provide.
     * @param {string} builder application builder token.
     * @param {string} [alias] application alias name
     */
    (provide: Registration<any> | symbol | string, builder?: Token<IApplication>, alias?: string): ClassDecorator;
    /**
     * DIModule decorator, use to define class as Application DIModule element.
     *
     * @DIModule
     */
    (target: Function): void;
}
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} decorType
 * @param {(Token<IModuleBuilder<T>> | IModuleBuilder<T>)} builder
 * @param {InjectToken<IApplication>} provideType default provide type.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
export declare function createDIModuleDecorator<T extends DIModuleMetadata>(decorType: string, builder: Token<IModuleBuilder<T>> | IModuleBuilder<T>, provideType?: Token<any>, adapter?: MetadataAdapter, metadataExtends?: MetadataExtends<T>): IDIModuleDecorator<T>;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
export declare const DIModule: IDIModuleDecorator<DIModuleMetadata>;
