"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
var IModuleBuilder_1 = require("../IModuleBuilder");
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} decorType
 * @param {(Token<IModuleBuilder<T>> | IModuleBuilder<T>)} builder
 * @param {InjectToken<IApplication>} provideType default provide type.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(decorType, builder, provideType, adapter, metadataExtends) {
    return core_1.createClassDecorator('DIModule', function (args) {
        if (adapter) {
            adapter(args);
        }
        args.next({
            match: function (arg) { return arg && (core_1.isString(arg) || (core_1.isObject(arg) && arg instanceof core_1.Registration)); },
            setMetadata: function (metadata, arg) {
                if (core_1.isString(arg)) {
                    metadata.name = arg;
                }
                else {
                    metadata.provide = arg;
                }
            }
        });
        args.next({
            match: function (arg) { return core_1.isString(arg) || core_1.isToken(arg); },
            setMetadata: function (metadata, arg) {
                if (core_1.isString(arg)) {
                    metadata.name = arg;
                }
                else {
                    metadata.builder = arg;
                }
            }
        });
        args.next({
            match: function (arg) { return core_1.isString(arg); },
            setMetadata: function (metadata, arg) {
                metadata.name = arg;
            }
        });
    }, function (metadata) {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.type)) {
            var isuglify = /^[a-z]$/.test(metadata.type.name);
            if (isuglify && metadata.type.classAnnations) {
                metadata.name = metadata.type.classAnnations.name;
            }
            else {
                metadata.name = metadata.type.name;
            }
        }
        metadata.provide = metadata.provide || provideType;
        metadata.alias = metadata.alias || metadata.name;
        metadata.decorType = decorType;
        if (!metadata.builder) {
            metadata.builder = builder;
        }
        return metadata;
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('', IModuleBuilder_1.ModuleBuilderToken);

//# sourceMappingURL=../sourcemaps/decorators/DIModule.js.map
